/* Copyright (C) 2015, Armin Biere, JKU Linz. */

/* Simple RATZ decode reading DRVT from stdin and writing DRAT to stdout.
 * This code should only illustrate the logic behind the variable-byte DRVT
 * format and probably lacks some error checking.
 */

#include <stdio.h>
#include <stdlib.h>

static int * lits, nlits, szlits;

static void parse_error (int id) {
  fprintf (stderr, "*** sztar: parse error %d\n", id);
  exit (1);
}

static void push_lit (int lit) {
  if (nlits == szlits) {
    if (szlits == (1<<30)) parse_error (1);
    szlits = szlits ? 2*szlits : 1;
    lits = realloc (lits, sizeof *lits * szlits);
  }
  lits[nlits++] = lit;
}

static unsigned read_word () {
  unsigned res = 0;
  int i, ch = 128;
  for (i = 0; ch > 127; i += 7)
    if ((ch = getc (stdin)) == EOF) parse_error (2);
    else res |= (ch & 127) << i;
  return res;
}

static int decode (int prev) {
  unsigned delta = read_word ();
  int res;
  if (!delta) return 0;
  res = abs (prev) + delta/2;
  if (delta & 1) res = -res;
  return res;
}

int main () {
  for (;;) {
    int ch = getc (stdin), lit = 0, i;
    unsigned pos;
    if (ch == EOF) return 0;
    ungetc (ch, stdin);
    pos = read_word ();
    if (pos & 2) parse_error (3);
    if (!(pos & 1)) fputs ("d ", stdout);
    pos >>= 2;
    while ((lit = decode (lit)))
      push_lit (lit);
    if (!nlits && pos) parse_error (4);
    if (nlits && pos >= nlits) parse_error (5);
    if (nlits) printf ("%d ", lits[pos]);
    for (i = 0; i < nlits; i++)
      if (i != (int) pos)
	printf ("%d ", lits[i]);
    fputs ("0\n", stdout);
    nlits = 0;
  }
  free (lits);
}
