#ifndef typez_h_INCLUDED
#define typez_h_INCLUDED

/*------------------------------------------------------------------------*/

#define RATZ_DEL_TYPE 0		/* 'delete' clause by contents */
#define RATZ_ADD_TYPE 1		/* 'add' clause */
#define RATZ_REL_TYPE 2		/* 'release' clause by index difference */
#define RATZ_SUB_TYPE 3		/* add 'subsuming' clause by index 
				 * difference and missing literals */
#define RATZ_TYPE_BITS 2
#define RATZ_TYPE_MASK ((1<<RATZ_TYPE_BITS)-1)

/*------------------------------------------------------------------------*/

/* These are only necessary for managing our alternative formats, more
 * precisely enabling differences and big-endian mode for the binary format
 * and disabling differences for the variable-byte format.  In production
 * use with the default variable-byte input format in difference mode there
 * is no header.  The trick is to use an impossible bit in the first byte of
 * the variable-byte format to mark the first byte a header and to
 * distinguish a legal first byte in the variable-byte format from a header
 * byte.  This illegal bit is taken to be the common bit set in 'release' or
 * in 'subsume' clauses, which both are impossible for a first clause, since
 * there is no other previous clause to release or subsume and thus it would
 * be a semantic not necassarily syntactic error to have a first non-header
 * byte with this bit set.
 */
#define RATZ_HEADER_BIT			RATZ_REL_TYPE

/*------------------------------------------------------------------------*/

#define RATZ_BINARY_HEADER_BIT		4
#define RATZ_NO_DIFFERENCES_HEADER_BIT	8
#define RATZ_BIG_ENDIAN_HEADER_BIT	16
#define RATZ_RELEASE_HEADER_BIT		32

#define RATZ_HEADER_MASK \
  ( RATZ_HEADER_BIT | \
    RATZ_BINARY_HEADER_BIT | \
    RATZ_NO_DIFFERENCES_HEADER_BIT | \
    RATZ_BIG_ENDIAN_HEADER_BIT | \
    RATZ_RELEASE_HEADER_BIT )

#define RATZ_MAKE_HEADER(BITS) \
  (assert (!((BITS) & ~RATZ_HEADER_MASK)), ((BITS) | RATZ_REL_TYPE))

/*------------------------------------------------------------------------*/

#endif
